Index: src/uk/me/parabola/mkgmap/osmstyle/RuleFileReader.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/RuleFileReader.java	(revision 3837)
+++ src/uk/me/parabola/mkgmap/osmstyle/RuleFileReader.java	(working copy)
@@ -127,10 +127,10 @@
 			if (scanner.isEndOfFile())
 				break;
 
-			Op expr = expressionReader.readConditions();
+			Op expr = expressionReader.readConditions(ifStack);
 			ActionList actionList = actionReader.readActions();
 			
-			expr = condExpr(expr, actionList);
+			checkIfStack(actionList);
 
 			// If there is an action list, then we don't need a type
 			GType type = null;
@@ -259,14 +259,16 @@
 		return true;
 	}
 
-	private Op condExpr(Op expr, ActionList actionList) {
-		Op result = expr;
-
+	/**
+	 * Check if one of the actions in the actionList would change the result of a previously read if expression.
+	 * If so, use the alternative expression with the generated tag.
+	 * @param actionList
+	 */
+	private void checkIfStack(ActionList actionList) {
+		if (actionList == null)
+			return;
 		for (Op[] ops : ifStack) {
-			AndOp and = new AndOp();
-			and.setFirst(result);
-			and.setSecond(ops[0]);
-			if (actionList != null && ops[0] != ops[1]) {
+			if (ops[0] != ops[1]) {
 				// check if this action can change the result of the initial if expression 
 				if (possiblyChanged(ops[0], actionList)) {
 					// the result may be changed, use the generated tag for all further rules in this if / else block
@@ -273,9 +275,7 @@
 					ops[0] = ops[1];
 				} 
 			}
-			result = and;
 		}
-		return result;
 	}
 
 	/**
Index: src/uk/me/parabola/mkgmap/osmstyle/eval/ExpressionReader.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/eval/ExpressionReader.java	(revision 3837)
+++ src/uk/me/parabola/mkgmap/osmstyle/eval/ExpressionReader.java	(working copy)
@@ -13,6 +13,8 @@
  */
 package uk.me.parabola.mkgmap.osmstyle.eval;
 
+import java.util.Collection;
+import java.util.Collections;
 import java.util.HashSet;
 import java.util.Set;
 import java.util.Stack;
@@ -52,6 +54,16 @@
 	 * or by end of file.
 	 */
 	public Op readConditions() {
+		return readConditions(Collections.emptyList());
+	}
+	
+	/**
+	 * Read the conditions.  They are terminated by a '[' or '{' character
+	 * or by end of file.
+	 * @param ifStack expressions of enclosing if / else 
+	 */
+	public Op readConditions(Collection <Op[]> ifStack) {
+		int startLine = scanner.getLinenumber();
 		while (!scanner.isEndOfFile()) {
 			scanner.skipSpace();
 			if (scanner.checkToken("[") || scanner.checkToken("{") || scanner.checkToken("then"))
@@ -84,9 +96,17 @@
 		while (!opStack.isEmpty())
 			runOp(scanner);
 
+		if (!ifStack.isEmpty() && stack.size() <= 1) {
+			Op op = null;
+			if (!stack.isEmpty())
+				op = stack.pop();
+			stack.push(appendIfExpr(op, ifStack));
+		}
+			
 		// The stack should contain one entry which is the complete tree
-		if (stack.size() != 1)
+		if (stack.size() != 1) {
 			throw new SyntaxException(scanner, "Stack size is "+stack.size());
+		}
 
 		assert stack.size() == 1;
 		Op op = stack.pop();
@@ -96,6 +116,27 @@
 	}
 
 	/**
+	 * Append previously read if/else expressions.
+	 * @param expr
+	 * @param ifStack
+	 * @return
+	 */
+	private Op appendIfExpr(Op expr, Collection<Op[]> ifStack) {
+		Op result = expr;
+		for (Op[] ops : ifStack) {
+			if (result != null) {
+				AndOp and = new AndOp();
+				and.setFirst(result);
+				and.setSecond(ops[0]);
+				result = and;
+			} else 
+				result = ops[0];
+			
+		}
+		return result;
+	}
+
+	/**
 	 * Is this a token representing an operation?
 	 * @param token The string to test.
 	 * @return True if this looks like an operator.
Index: test/resources/rules/if-then-2.test
===================================================================
--- test/resources/rules/if-then-2.test	(nonexistent)
+++ test/resources/rules/if-then-2.test	(working copy)
@@ -0,0 +1,19 @@
+
+#
+# Test for an if where a rule modifies the if expression
+#
+
+WAY 1
+A=1
+B=1
+
+<<<lines>>>
+
+if (B=1) then
+   {set C=1}
+end
+C=1	[0x1 resolution 24]
+ 
+<<<results>>>
+NO-STRICT
+WAY 1: Line 0x1, labels=[null, null, null, null], res=24-24 (1/1),(2/2),
